@@ -13,7 +13,7 @@ use File::Temp 'tempfile';
 use FindBin '$Bin';
 use Test::Builder;
 
-our $VERSION = '0.10_02';
+our $VERSION = '0.10_03';
 
 sub new
 {
@@ -114,8 +114,9 @@ sub expect_charset
     eval "use Unicode::UTF8 qw/decode_utf8 encode_utf8/";
     if ($@) {
 	Encode->import (qw/decode_utf8 encode_utf8/);
-	if (! $self->{no_warn}) {
-	    carp "No Unicode::UTF8, falling back to Encode";
+	if (! $self->{no_warn} && ! $self->{_warned_unicode_utf8}) {
+	    carp "Unicode::UTF8 is not installed, using Encode";
+	    $self->{_warned_unicode_utf8} = 1;
 	}
     }
     $self->note ("You have told me to expect a 'charset' value of '$charset'.");
@@ -287,11 +288,18 @@ sub setenv_private
 
 sub encode_utf8_safe
 {
+    my ($self) = @_;
+    my $input = $self->{input};
     eval "use Unicode::UTF8;";
     if ($@) {
-	return Encode::encode_utf8 (@_);
+	if (! $self->{no_warn} && ! $self->{_warned_unicode_utf8}) {
+	    carp "Unicode::UTF8 is not installed, using Encode";
+	    $self->{_warned_unicode_utf8} = 1;
+	}
+	# Encode::encode_utf8 uses prototypes so we have to hassle this up.
+	return Encode::encode_utf8 ($input);
     }
-    return Unicode::UTF8::encode_utf8 (@_);
+    return Unicode::UTF8::encode_utf8 ($input);
 }
 
 # Internal routine to run a CGI program.
@@ -340,7 +348,7 @@ sub run_private
     if (defined $options->{input}) {
         $self->{input} = $options->{input};
 	if (utf8::is_utf8 ($self->{input})) {
-	    $self->{input} = encode_utf8_safe ($self->{input});
+	    $self->{input} = $self->encode_utf8_safe ();
 	}
         my $content_length = length ($self->{input});
         setenv_private ($self, 'CONTENT_LENGTH', $content_length);
